require 'studiogame/spechelper'
require 'studiogame/player'

module StudioGame
  describe Player do

    before do
      $stdout = StringIO.new
      @initial_health = 150
      @player = Player.new("larry", @initial_health)
    end

      #capitalized name
      it "has a capitalized name" do
        @player.name.should == "Larry"
        #--------------------------------------------------------------rspec 3 w/o the spechelper file down below
        #------------------------------------------------------------- expect(player.name).to eq("Larry")
      end

      #initial health
      it "has an initial health" do
        @player.health.should == 150
      end

      it "has a string representation" do
        @player.found_treasure(Treasure.new(:hammer, 50))
        @player.found_treasure(Treasure.new(:hammer, 50))
        @player.to_s.should == "I'm Larry with health = 150, points = 100, and score = 250."
      end

      #score
      it "computes a score as the sum of its health and points" do
        @player.found_treasure(Treasure.new(:hammer, 50))
        @player.found_treasure(Treasure.new(:hammer, 50))
        @player.score.should == 250
      end

      #healthpack method
      it "increases health by 10 when healthpacked" do
        @player.healthpack
        @player.health.should == @initial_health+10
      end

      #hit method
      it "decreases health by 10 when hit" do
        @player.hit
        @player.health.should == @initial_health-10
      end

      #strong
      context "with a health of at least 100" do
        before do
          @player = Player.new("larry", 100)
        end
        it "is a strong" do
          @player.should be_strong
        end
        it "has a strong status" do
          @player.status.should == "strong"
        end
      end

      #weak
      context "with a health of less than 100" do
        before do
          @player = Player.new("larry", 90)
        end
        it "is not strong" do
          @player.should_not be_strong
        end
        it "has a weak status" do
          @player.status.should == "weak"
        end
      end


        #Decending order
      context "in a collection of players" do
        before do
          @player1 = Player.new("moe", 100)
          @player2 = Player.new("larry", 200)
          @player3 = Player.new("curly", 300)

          @players = [@player1, @player2, @player3]

          it "is sorted by decending order" do
            @players.sort.should == [@player3, @player2, @player1]
          end
        end
      end

      it "computes as the sum of all treasure points" do

        @player.found_treasure(Treasure.new(:hammer, 50))
        @player.found_treasure(Treasure.new(:hammer, 50))

        @player.points.should == 100

      end

      it "yields each found treasure and its total points" do
        @player.found_treasure(Treasure.new(:skillet, 100))
        @player.found_treasure(Treasure.new(:skillet, 100))
        @player.found_treasure(Treasure.new(:hammer, 50))
        @player.found_treasure(Treasure.new(:bottle, 5))
        @player.found_treasure(Treasure.new(:bottle, 5))
        @player.found_treasure(Treasure.new(:bottle, 5))
        @player.found_treasure(Treasure.new(:bottle, 5))
        @player.found_treasure(Treasure.new(:bottle, 5))

        yielded = []
        @player.each_found_treasure do |treasure|
          yielded << treasure
        end

        yielded.should == [
          Treasure.new(:skillet, 200),
          Treasure.new(:hammer, 50),
          Treasure.new(:bottle, 25)
       ]
      end

      it "can be created from a CSV string" do
      player = Player.from_csv("larry,150")

      player.name.should == "Larry"
      player.health.should == 150
    end
  end
end
